*:*****************************************************************************
*:
*: Procedure file: J:\TREE\TREE.PRG
*:
*:         System: Directory Recursion with Graphical Tree
*:         Author: Phil Barnett
*:      Copyright (c) 1994, Public Domain
*:  Last modified: 09/14/94      9:33
*:
*:  Procs & Fncts: TREE()
*:               : DIR_RECURS
*:               : Branch()
*:               : SHOW1
*:               : SHOW2
*:
*:      Documented 09/14/94 at 09:52                SNAP!  version 5.02
*:*****************************************************************************
#INCLUDE "box.ch"
#INCLUDE "directry.ch"
#INCLUDE "common.ch"

static aLevel := {}, nLevel, nHandle

*!*****************************************************************************
*!
*!       Function: TREE()
*!
*!          Calls: SHOW2              (procedure in TREE.PRG)
*!               : DIR_RECURS         (procedure in TREE.PRG)
*!               : BRANCH()           (function  in TREE.PRG)
*!
*!           Uses: FILES.DBF          
*!               : TREE.DBF           
*!
*!        Indexes: PATH.NTX           
*!               : TREEPATH.NTX       
*!               : PARENT.NTX         
*!
*!    Other Files: OUTPUT.TXT
*!
*!*****************************************************************************
function TREE( cDrive, cTreeFile )

local cParentDir, nParent, nCurrentRec

field path, NAME, size, FDAT, FTIM, ATTR, PARENT

default cDrive to 'C:'
default cTreeFile to 'TREE.TXT'

cDrive := upper( cDrive )
cTreeFile := upper( cTreeFile )

setcolor( 'G+/B' )
clear screen

@ 3,24 say 'Directory Walker and Tree Creator'

dispbox(  5, 5, 13, 75, B_SINGLE + ' ' )
dispbox( 14, 5, 22, 75, B_SINGLE + ' ' )
setcolor( 'GR+/B' )

dbcreate('files.dbf',{ { 'PATH', 'C', 40, 0 },;
                       { 'NAME', 'C', 12, 0 },;
                       { 'SIZE', 'N', 10, 0 },;
                       { 'FDAT', 'D',  8, 0 },;
                       { 'FTIM', 'C',  8, 0 },;
                       { 'ATTR', 'C',  4, 0 } } )

dbcreate('tree.dbf',{ { 'PATH',   'C', 40, 0 },;
                      { 'COUNT',  'N', 10, 0 },;
                      { 'SIZE',   'N', 10, 0 },;
                      { 'PARENT', 'N',  8, 0 } } )

use files
// index on path to path
SHOW2( 'Scanning Directories on ' + cDrive )
DIR_RECURS( cDrive )

use TREE NEW
index on path to TREEPATH

select files
dbgotop()
do while !eof()
   select TREE
   if !dbseek( FILES->PATH )
      dbappend()
      TREE->PATH := FILES->PATH
      SHOW2( TREE->PATH )
   endif
   if !( 'D' $ FILES->ATTR )
      TREE->COUNT ++
      TREE->SIZE += FILES->SIZE
   endif
   select files
   skip
enddo

select TREE
go top
skip
do while !eof()
   cParentDir := left( path, rat( '\', path ) - 1 )
   nCurrentRec := recno()
   dbseek( cParentDir )
   nParent := recno()
   dbgoto( nCurrentRec )
   TREE->PARENT := nParent
   dbskip()
enddo

index on PARENT to PARENT

asize( aLevel, 30 )
afill( aLevel, .f. )
nLevel := 0
go top

nHandle := fcreate( cTreeFile, 0 )

Branch()

SHOW2( 'Tree text file is: ' + cTreeFile )

fclose( nHandle )

dbcloseall()

devpos( 22, 0 )

return NIL

*!*****************************************************************************
*!
*!      Procedure: DIR_RECURS
*!
*!      Called by: TREE()             (function  in TREE.PRG)
*!               : DIR_RECURS         (procedure in TREE.PRG)
*!
*!          Calls: DIR_RECURS         (procedure in TREE.PRG)
*!
*!*****************************************************************************
static function DIR_RECURS( cPath )

local x
local aFiles   := directory( cPath + '\*.*', 'D' )
local nFilCount := len( aFiles )

for x := 1 to nFilCount
   if aFiles[ X, F_NAME ] <> '..'
      dbappend()
      FILES->PATH := cPath
      FILES->NAME := aFiles[ X, F_NAME ]
      FILES->SIZE := aFiles[ X, F_SIZE ]
      FILES->FDAT := aFiles[ X, F_DATE ]
      FILES->FTIM := aFiles[ X, F_TIME ]
      FILES->ATTR := aFiles[ X, F_ATTR ]
      SHOW1( cPath + '\' + aFiles[ X, 1 ] )
   endif
   if 'D' $ aFiles[ X, F_ATTR ]
      if aFiles[ X, F_NAME ] <> '.'
         DIR_RECURS( cPath + '\' + aFiles[ X, F_NAME ] )
      endif
   endif
next

return NIL

*!*****************************************************************************
*!
*!       Function: BRANCH()
*!
*!      Called by: TREE()             (function  in TREE.PRG)
*!               : BRANCH()           (function  in TREE.PRG)
*!
*!          Calls: SHOW2              (procedure in TREE.PRG)
*!               : BRANCH()           (function  in TREE.PRG)
*!
*!*****************************************************************************
function Branch()

#define nIndent 2

local nParent, nCurrentRec, nNode, cThisLine

field PARENT, size, count, path

nLevel ++
cThisLine := transform( count, "9,999" ) + ' ' + transform( size, "999,999,999") + ' '
if nLevel > 1
   for nNode := 2 to nLevel - 1
      cThisLine += iif( aLevel[ nNode ], ' ', '' ) + space( nIndent )
   next
   nCurrentRec := recno()
   nParent := PARENT
   skip
   if nParent == PARENT
      cThisLine += '' + replicate( '', nIndent )
      aLevel[ nLevel ] = .f.
   else
      cThisLine += '' + replicate( '', nIndent )
      aLevel[ nLevel ] = .t.
   endif
   dbgoto( nCurrentRec )
endif
cThisLine += iif( len( alltrim( path ) ) = 2, path, substr( path, rat( '\', path ) + 1 ) )

SHOW2( cThisLine )

fwrite( nHandle, cThisLine + chr( 13 ) + chr( 10 ) )

nParent := recno()
dbseek( nParent )
do while PARENT == nParent
   nCurrentRec := recno()
   Branch()
   dbgoto( nCurrentRec )
   skip
enddo

nLevel --

return NIL

*!*****************************************************************************
*!
*!      Procedure: SHOW1
*!
*!*****************************************************************************
static function SHOW1( text )

scroll( 6, 6, 12, 74, 1 )

@ 12, 6 say text

return NIL

*!*****************************************************************************
*!
*!      Procedure: SHOW2
*!
*!      Called by: TREE()             (function  in TREE.PRG)
*!               : Branch()         (function  in TREE.PRG)
*!
*!*****************************************************************************
static function SHOW2( text )

scroll( 15, 6, 21, 74, 1 )

@ 21, 6 say text

return NIL

*: EOF: TREE.PRG
